'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _yosay = require('yosay');

var _yosay2 = _interopRequireDefault(_yosay);

var _ejs = require('ejs');

var _ejs2 = _interopRequireDefault(_ejs);

var _makeup = require('./makeup');

var _makeup2 = _interopRequireDefault(_makeup);

var _base = require('../../lib/base');

var _base2 = _interopRequireDefault(_base);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class AppGenerator extends _base2.default {

  constructor(...args) {
    super(...args);
    // console.log('AppGenerator appname=', this.appname);
    this.sourceRoot(_path2.default.join(__dirname, '../../../templates/app'));
    this._setupMakeup();
  }

  _setupMakeup() {
    const makeup = new _makeup2.default(this);
    makeup.applyConfig(this.config.getAll());
    makeup.applyOptions(this.options);
    makeup.applyDefaults(this.appname);
    this.makeup = makeup;
  }

  prompting() {
    if (!this.options['skip-welcome-message']) {
      // Have Yeoman greet the user.
      this.log((0, _yosay2.default)('Welcome to the marvelous WebpackVersatile generator!'));
    }

    let questions = this.makeup.getQuestions();
    return this.prompt(questions).then(answers => {
      this.makeup.putAnswers(answers);
    });
  }

  configuring() {
    this.config.set(this.makeup.getPermanetConfig());
    this.config.save();
  }

  writing() {
    const templateConfig = this.makeup.getTemplateConfig();
    const dirs = templateConfig.dirs;

    const templateTransformer = {
      src: /^%(.*)/,
      tgt: '$1',
      fn: s => {
        return _ejs2.default.render(s, templateConfig);
      }
    };

    // this.log('copy project files');
    this._branchCopy({
      source: 'root',
      branches: templateConfig.use,
      transformers: [templateTransformer]
    });

    // this.log('copy src files');
    this._branchCopy({
      source: 'src',
      target: dirs.src,
      branches: templateConfig.use,
      transformers: [templateTransformer]
    });

    // this.log('copy test files');
    this._branchCopy({
      source: 'test',
      target: dirs.test,
      branches: templateConfig.use,
      transformers: [templateTransformer]
    });
  }

  install() {
    if (!this.options['skip-install']) {
      this.installDependencies();
    }
  }
}

exports.default = AppGenerator;